unit BGRAmoon1;

(*
MoonPhase
DE
-100 = vollmond
-75 = abnemend links kreis - rechts weniger
-50 = halbmond    links hell - rechts dunkel
-25 = abnehmend links kreis sichel   =  (
0 = neumond
+25 zunehmend rechts kreis sichel
+50 halbmond zunehmend links dunkel - rechts hell
+75 rechts kreis - links weniger   = )
+-100 vollmond

EN
-100 fullmoon
-75 waning circle left
-50 half moon, left bright right dark
-25 waning, circle left, crecent
0 new moon
+25 waxing, circle right crescent
+50 half moon, left dark right bright
+75 waxing, cirle right
+-100 full moon
*)

{$mode objfpc}{$H+}

interface

uses
   Classes, SysUtils, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls,
	 Buttons, math, LCLtype, BGRAbitmap, BGRABitmapTypes, BGRAClasses;

type

	 { TForm1 }

   TForm1 = class(TForm)
			Label1: TLabel;
			Sp1: TSpeedButton;
			Timer1: TTimer;
   procedure CreateCircle;
   procedure CreateMoon;
   procedure DrawMoon;
   procedure FormActivate(Sender: TObject);
   procedure FormCreate(Sender: TObject);
   procedure FormPaint(Sender: TObject);
   procedure FormResize(Sender: TObject);
   procedure Sp1Click(Sender: TObject);
   procedure Timer1Timer(Sender: TObject);
   private
   var moon,circle : ArrayOfTPointF;
   phase, radius : single;
   center : TPointF;
   DateOffset : TDateTime; // last full moon before now
   public

   end;

var
   Form1: TForm1;

implementation
{$R *.lfm}

const deg2rad = pi/180;
SynMoon = 29.53; // days average

var startup : Boolean = true;

(*  Done now by BGRAClasses:
procedure incF (var i : single; increment: single = 1.0); inline;
begin
i := i + increment;
end;
*)

// sinus/cosinus vals for every degree to speed up calculation
// for easy getting left/right half starting at bottom (90°)
procedure TForm1.CreateCircle;
var  sinus, cosinus : single;
     angle : integer;
begin
setLength(circle,361);
for angle := 90 to 360 + 90 do
  begin
  sincos (angle*deg2Rad,sinus,cosinus);
  circle[angle-90] := PointF (cosinus,sinus);
	end;
end;


procedure TForm1.CreateMoon ;
var size,i : integer;
    len : single;
    sign : integer;
begin
center := PointF (Width/2,height/2);
size := math.min(width,height);
radius := (size - 8 ) / 2;
if phase < 0 then
   begin
	    moon := copy (circle,0,181); // left half, waning
	    sign := +1;
   end else
   begin
	    sign := -1;
	    moon := copy (circle ,180,181); // right half, waxing
	 end;

SetLength(moon,360);
for i := 179 downto 1 do
  begin
  len := moon[i].x*2*radius*phase/100 * sign;
  moon[360-i] := PointF(moon[i].x*radius+len,moon[i].y*radius)+center;
	end;
for i := 0 to 180 do moon[i] := PointF(moon[i].x*radius, Moon[i].y * radius) +center; ;
end;


procedure TForm1.DrawMoon;
var tmp: TBGRAbitmap;
    w : integer;
    R: Trect;
 begin
if startUp then exit;
tmp:= TBGRAbitmap.create (width, height, cssBlue);
tmp.FillEllipseAntialias(Center.x,Center.y,radius,radius,CSSDarkBlue);
Tmp.DrawPolygonAntialias(moon,cssBlack,0.5,CSSYellow);
tmp.draw(canvas,0,0);
application.ProcessMessages;
// Icons
if abs(trunc(phase*2)) mod 5 = 0 then
  begin
  tmp.EllipseAntialias(Center.x,center.y,radius-4,radius-4,cssBlack,9);
  tmp.FloodFill(0,0,BGRAPixelTransparent,fmProgressive);
  w := trunc(radius)+4;
  R := Rect (trunc(Center.x-w),trunc(Center.y-w),trunc(Center.x+w),
             trunc(Center.y+w));
  BGRAreplace (Tmp,tmp.getPart(R));
  BGRAreplace (tmp,tmp.resample (icon.width,icon.height));
  R := Rect(0,0,tmp.width,tmp.height);
  Application.Icon.Canvas.CopyRect(R,tmp.canvas,R);
  (* Caveat:
  a) Canvas.CopyRect on the form.icon is blocking with Window: app frozen
  b) BGRAbitmap.draw does nothing to the Form.icon.canvas and to the
     Application.Icon.Canvas with Linux
  *)
  {$IFDEF LINUX} self.Icon.Canvas.CopyRect(R,tmp.canvas,R);{$endif}
  {$IFDEF Windows} tmp.draw(self.icon.Canvas,0,0); {$endif}
  end;
tmp.free;
end;

procedure TForm1.FormActivate(Sender: TObject);
begin
   startup := false;
   OnActivate:= nil;
end;



function Range2Word (i: integer) : word; inline;
begin
 if i > MAXWORD then i := MAXWORD else
       if i < 0 then i := 0;
result := i;
end;

Function ResizeRectF (R: TRectF; delta: single): TRectF;
begin
with result do
 begin
 Left := R.Left+delta;
 Right :=R.right - delta;
 Top := R.Top + delta;
 Bottom := R.Bottom - delta;
 end;

end;

procedure BevelRect (tmp: TBGRAbitmap; R: TRectF; Border: single; Col: TBGRAPixel);
const lightShift = 64;
var poly : array of TPointF;
    Rinner: TRectF;
		hsl: THSLAPixel;
    Dark, Light : TBGRAPixel;
begin
hsl := BGRAToHSLA(Col);
Light := HSLAToBGRA(HSLA(hsl.hue,hsl.saturation,Range2Word(hsl.lightness+lightShift*256)));
Dark := HSLAToBGRA(HSLA(hsl.hue,hsl.saturation,Range2Word(hsl.lightness-lightShift*256)));
Setlength(poly,5);
Poly[0] := PointF(R.left,R.Bottom);
Poly[1] := PointF (R.left+border,R.bottom-Border);
Poly[2] := PointF (R.Right-border,R.Top+Border);
Poly[3] := PointF (R.Right,R.Top);
Poly[4] := PointF (R.Right,R.Bottom);
tmp.FillPolyAntialias(Poly,Dark);
tmp.FloodFill(0,0,light,fmProgressive);
RInner := ResizeRectF(R,border);
Tmp.FillRectAntialias(Rinner,col);
tmp.DrawLineAntialias(R.Left,R.Top,Rinner.left,Rinner.Top,col,1);
tmp.DrawLineAntialias(R.Right,R.Bottom,Rinner.Right,RInner.Bottom,col,0.5);
end;

procedure TForm1.FormCreate(Sender: TObject);
var tmp : TBGRAbitmap;
	  R:TrectF;
    RInt : Trect;
    border: single;
    FullMoon : TDateTime;

begin
 phase := -100.5;
 createCircle;
// Vollmond 28. März 2021	20:48
 FullMoon := EncodeDate(2021,3,28)+EncodeTime(20,48,0,0);
 DateOffset := FullMoon;
 While DateOffset +SynMoon < Now do
    DateOffset := DateOffset+ SynMoon;

 Sp1.Glyph.SetSize(Sp1.Width,Sp1.Height);
 //Sp1.Glyph.Canvas.Brush.Color := clBlack;
 //Sp1.Glyph.Canvas.FillRect(0,0,Sp1.width,Sp1.Height);
 tmp := TBGRAbitmap.create (Sp1.Width,Sp1.Height,cssWhite);
 R := RectF(0,0,Sp1.Width,Sp1.Height);
 Border := Sp1.Height/5;
 BevelRect(tmp,R,border,CSSForestGreen);
 tmp.FontHeight:= 13;
 tmp.FontStyle:= [fsBold];
 tmp.textOut (tmp.width/2,border,'EXIT', CSSWhite,taCenter);
 RInt := Rect(0,0,trunc(R.Right),trunc(R.Bottom) );
 Sp1.Glyph.canvas.copyRect(Rint,tmp.Canvas,Rint);
 tmp.free;
 Label1.Font.Style := [fsBold];
 Label1.Font.Color:= clWhite;

end;

procedure TForm1.FormPaint(Sender: TObject);
begin
   DrawMoon;
end;

procedure TForm1.FormResize(Sender: TObject);
var size, FSize: Integer;
begin
   size := min(Width,Height);
   if size > 450 then Fsize :=14 else
      begin
      if size > 350 then  FSize := 12 else FSize :=  10;
			end;
   Label1.Font.size := FSize;
   // Caption := IntToStr(Width)+' x '+IntToStr(Height);
end;

procedure TForm1.Sp1Click(Sender: TObject);
begin
   close;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var D : TDateTime;
    tmp : Double;
begin
CreateMoon;
tmp := (phase +100) / 200;
if tmp >= 1.0 then tmp := 0;
D := DateOffset + SynMoon*tmp;
Label1.Caption := #32 +formatdatetime('yyyy·mm·dd', D) +#32;
incF (phase,0.5);
if phase >= 100 then phase := -100;
invalidate;
end;

end.

